import base64
import os

from PIL import Image
from selenium.webdriver.common.keys import Keys

from .captcha import Captcha

from ..exceptions import ParamsException


class ImageCaptcha(Captcha):
    def __init__(self, **params):
        super().__init__(**params)

        if not self.params["image_xpath"]:
            raise ParamsException(
                "No image_xpath provided. Cannot work witouht image with captcha!"
            )

        self.image_jpg_path = os.path.join(os.getcwd(), "captcha_image.jpg")

    def solve(self):
        self._get_captcha_img()
        self._encode_image()
        self.api_provider.post_in(self.body)
        self.token = self.api_provider.get_res()

        if self.params["upper"]:
            self.token = self.token.upper()

        if self.params["input_xpath"]:
            self._input_captcha()

        if self.params["click_xpath"]:
            self.click_solve_captcha()

        if self.params["check_xpath"]:
            self.check_captcha()

        return True

    def _get_captcha_img(self):
        image_png_path = os.path.join(os.getcwd(), "captcha_image.png")
        self.browser.wait_until_page_contains_element(
            self.params["image_xpath"], timeout=5
        )
        with open(image_png_path, "wb") as f:
            l = self.browser.find_element(self.params["image_xpath"])
            f.write(l.screenshot_as_png)

        img = Image.open(image_png_path)
        rgb_im = img.convert("RGB")
        rgb_im.save(self.image_jpg_path, quality=10)
        os.remove(image_png_path)

    def _encode_image(self):
        with open(self.image_jpg_path, "rb") as f:
            self.body = base64.b64encode(f.read())
        os.remove(self.image_jpg_path)

    def _input_captcha(self):
        self.browser.click_element_when_visible(self.params["input_xpath"])

        for d in self.token:
            self.browser.press_keys(str(d))
        self.browser.press_keys(Keys.ENTER)
